@@ -4,26 +4,26 @@
 
     File utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import posixpath
+from typing import Dict
 
 from docutils.utils import relative_path
 
 from sphinx.util.osutil import copyfile, ensuredir
+from sphinx.util.typing import PathMatcher
 
 if False:
     # For type annotation
-    from typing import Callable, Dict, Union  # NOQA
-    from sphinx.util.matching import Matcher  # NOQA
-    from sphinx.util.template import BaseRenderer  # NOQA
+    from sphinx.util.template import BaseRenderer
 
 
-def copy_asset_file(source, destination, context=None, renderer=None):
-    # type: (str, str, Dict, BaseRenderer) -> None
+def copy_asset_file(source: str, destination: str,
+                    context: Dict = None, renderer: "BaseRenderer" = None) -> None:
     """Copy an asset file to destination.
 
     On copying, it expands the template variables if context argument is given and
@@ -55,8 +55,8 @@ def copy_asset_file(source, destination, context=None, renderer=None):
         copyfile(source, destination)
 
 
-def copy_asset(source, destination, excluded=lambda path: False, context=None, renderer=None):
-    # type: (str, str, Union[Callable[[str], bool], Matcher], Dict, BaseRenderer) -> None
+def copy_asset(source: str, destination: str, excluded: PathMatcher = lambda path: False,
+               context: Dict = None, renderer: "BaseRenderer" = None) -> None:
     """Copy asset files to destination recursively.
 
     On copying, it expands the template variables if context argument is given and
