@@ -31,7 +31,7 @@ if False:
     from sphinx.builders import Builder
     from sphinx.domain import IndexEntry
     from sphinx.environment import BuildEnvironment
-    from sphinx.utils.tags import Tags
+    from sphinx.util.tags import Tags
 
 logger = logging.getLogger(__name__)
 
