@@ -4,22 +4,24 @@
 
     Experimental docutils writers for HTML5 handling Sphinx' custom nodes.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import posixpath
-import sys
+import re
 import warnings
-from typing import Iterable, cast
+from typing import Any, Iterable, Tuple
+from typing import cast
 
 from docutils import nodes
+from docutils.nodes import Element, Node, Text
 from docutils.writers.html5_polyglot import HTMLTranslator as BaseTranslator
 
 from sphinx import addnodes
 from sphinx.builders import Builder
-from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import admonitionlabels, _, __
 from sphinx.util import logging
 from sphinx.util.docutils import SphinxTranslator
@@ -27,8 +29,7 @@ from sphinx.util.images import get_image_size
 
 if False:
     # For type annotation
-    from typing import Any  # NOQA
-    from sphinx.builders.html import StandaloneHTMLBuilder  # NOQA
+    from sphinx.builders.html import StandaloneHTMLBuilder
 
 
 logger = logging.getLogger(__name__)
@@ -37,6 +38,19 @@ logger = logging.getLogger(__name__)
 # http://www.arnebrodowski.de/blog/write-your-own-restructuredtext-writer.html
 
 
+def multiply_length(length: str, scale: int) -> str:
+    """Multiply *length* (width or height) by *scale*."""
+    matched = re.match(r'^(\d*\.?\d*)\s*(\S*)$', length)
+    if not matched:
+        return length
+    elif scale == 100:
+        return length
+    else:
+        amount, unit = matched.groups()
+        result = float(amount) * scale / 100
+        return "%s%s" % (int(result), unit)
+
+
 class HTML5Translator(SphinxTranslator, BaseTranslator):
     """
     Our custom HTML translator.
@@ -44,8 +58,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
 
     builder = None  # type: StandaloneHTMLBuilder
 
-    def __init__(self, *args):
-        # type: (Any) -> None
+    def __init__(self, *args: Any) -> None:
         if isinstance(args[0], nodes.document) and isinstance(args[1], Builder):
             document, builder = args
         else:
@@ -62,7 +75,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         self.permalink_text = self.config.html_add_permalinks
         # support backwards-compatible setting to a bool
         if not isinstance(self.permalink_text, str):
-            self.permalink_text = self.permalink_text and '¶' or ''
+            self.permalink_text = '¶' if self.permalink_text else ''
         self.permalink_text = self.encode(self.permalink_text)
         self.secnumber_suffix = self.config.html_secnumber_suffix
         self.param_separator = ''
@@ -71,84 +84,63 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         self._fieldlist_row_index = 0
         self.required_params_left = 0
 
-    def visit_start_of_file(self, node):
-        # type: (nodes.Element) -> None
+    def visit_start_of_file(self, node: Element) -> None:
         # only occurs in the single-file builder
         self.docnames.append(node['docname'])
         self.body.append('<span id="document-%s"></span>' % node['docname'])
 
-    def depart_start_of_file(self, node):
-        # type: (nodes.Element) -> None
+    def depart_start_of_file(self, node: Element) -> None:
         self.docnames.pop()
 
-    def visit_desc(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'dl', CLASS=node['objtype']))
 
-    def depart_desc(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc(self, node: Element) -> None:
         self.body.append('</dl>\n\n')
 
-    def visit_desc_signature(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_signature(self, node: Element) -> None:
         # the id is set automatically
         self.body.append(self.starttag(node, 'dt'))
-        # anchor for per-desc interactive data
-        if node.parent['objtype'] != 'describe' \
-           and node['ids'] and node['first']:
-            self.body.append('<!--[%s]-->' % node['ids'][0])
 
-    def depart_desc_signature(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_signature(self, node: Element) -> None:
         if not node.get('is_multiline'):
             self.add_permalink_ref(node, _('Permalink to this definition'))
         self.body.append('</dt>\n')
 
-    def visit_desc_signature_line(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_signature_line(self, node: Element) -> None:
         pass
 
-    def depart_desc_signature_line(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_signature_line(self, node: Element) -> None:
         if node.get('add_permalink'):
             # the permalink info is on the parent desc_signature node
             self.add_permalink_ref(node.parent, _('Permalink to this definition'))
         self.body.append('<br />')
 
-    def visit_desc_addname(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_addname(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'code', '', CLASS='sig-prename descclassname'))
 
-    def depart_desc_addname(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_addname(self, node: Element) -> None:
         self.body.append('</code>')
 
-    def visit_desc_type(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_type(self, node: Element) -> None:
         pass
 
-    def depart_desc_type(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_type(self, node: Element) -> None:
         pass
 
-    def visit_desc_returns(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_returns(self, node: Element) -> None:
         self.body.append(' &#x2192; ')
 
-    def depart_desc_returns(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_returns(self, node: Element) -> None:
         pass
 
-    def visit_desc_name(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_name(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'code', '', CLASS='sig-name descname'))
 
-    def depart_desc_name(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_name(self, node: Element) -> None:
         self.body.append('</code>')
 
-    def visit_desc_parameterlist(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_parameterlist(self, node: Element) -> None:
         self.body.append('<span class="sig-paren">(</span>')
         self.first_param = 1
         self.optional_param_level = 0
@@ -157,8 +149,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
                                          for c in node.children])
         self.param_separator = node.child_text_separator
 
-    def depart_desc_parameterlist(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_parameterlist(self, node: Element) -> None:
         self.body.append('<span class="sig-paren">)</span>')
 
     # If required parameters are still to come, then put the comma after
@@ -167,8 +158,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
     #
     #     foo([a, ]b, c[, d])
     #
-    def visit_desc_parameter(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_parameter(self, node: Element) -> None:
         if self.first_param:
             self.first_param = 0
         elif not self.required_params_left:
@@ -178,50 +168,40 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         if not node.hasattr('noemph'):
             self.body.append('<em class="sig-param">')
 
-    def depart_desc_parameter(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_parameter(self, node: Element) -> None:
         if not node.hasattr('noemph'):
             self.body.append('</em>')
         if self.required_params_left:
             self.body.append(self.param_separator)
 
-    def visit_desc_optional(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_optional(self, node: Element) -> None:
         self.optional_param_level += 1
         self.body.append('<span class="optional">[</span>')
 
-    def depart_desc_optional(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_optional(self, node: Element) -> None:
         self.optional_param_level -= 1
         self.body.append('<span class="optional">]</span>')
 
-    def visit_desc_annotation(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_annotation(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'em', '', CLASS='property'))
 
-    def depart_desc_annotation(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_annotation(self, node: Element) -> None:
         self.body.append('</em>')
 
-    def visit_desc_content(self, node):
-        # type: (nodes.Element) -> None
+    def visit_desc_content(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'dd', ''))
 
-    def depart_desc_content(self, node):
-        # type: (nodes.Element) -> None
+    def depart_desc_content(self, node: Element) -> None:
         self.body.append('</dd>')
 
-    def visit_versionmodified(self, node):
-        # type: (nodes.Element) -> None
+    def visit_versionmodified(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'div', CLASS=node['type']))
 
-    def depart_versionmodified(self, node):
-        # type: (nodes.Element) -> None
+    def depart_versionmodified(self, node: Element) -> None:
         self.body.append('</div>\n')
 
     # overwritten
-    def visit_reference(self, node):
-        # type: (nodes.Element) -> None
+    def visit_reference(self, node: Element) -> None:
         atts = {'class': 'reference'}
         if node.get('internal') or 'refuri' not in node:
             atts['class'] += ' internal'
@@ -249,41 +229,34 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
             self.body.append(('%s' + self.secnumber_suffix) %
                              '.'.join(map(str, node['secnumber'])))
 
-    def visit_number_reference(self, node):
-        # type: (nodes.Element) -> None
+    def visit_number_reference(self, node: Element) -> None:
         self.visit_reference(node)
 
-    def depart_number_reference(self, node):
-        # type: (nodes.Element) -> None
+    def depart_number_reference(self, node: Element) -> None:
         self.depart_reference(node)
 
     # overwritten -- we don't want source comments to show up in the HTML
-    def visit_comment(self, node):  # type: ignore
-        # type: (nodes.Element) -> None
+    def visit_comment(self, node: Element) -> None:  # type: ignore
         raise nodes.SkipNode
 
     # overwritten
-    def visit_admonition(self, node, name=''):
-        # type: (nodes.Element, str) -> None
+    def visit_admonition(self, node: Element, name: str = '') -> None:
         self.body.append(self.starttag(
             node, 'div', CLASS=('admonition ' + name)))
         if name:
             node.insert(0, nodes.title(name, admonitionlabels[name]))
 
-    def visit_seealso(self, node):
-        # type: (nodes.Element) -> None
+    def visit_seealso(self, node: Element) -> None:
         self.visit_admonition(node, 'seealso')
 
-    def depart_seealso(self, node):
-        # type: (nodes.Element) -> None
+    def depart_seealso(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def add_secnumber(self, node):
-        # type: (nodes.Element) -> None
+    def get_secnumber(self, node: Element) -> Tuple[int, ...]:
         if node.get('secnumber'):
-            self.body.append('.'.join(map(str, node['secnumber'])) +
-                             self.secnumber_suffix)
-        elif isinstance(node.parent, nodes.section):
+            return node['secnumber']
+
+        if isinstance(node.parent, nodes.section):
             if self.builder.name == 'singlehtml':
                 docname = self.docnames[-1]
                 anchorname = "%s/#%s" % (docname, node.parent['ids'][0])
@@ -293,15 +266,20 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
                 anchorname = '#' + node.parent['ids'][0]
                 if anchorname not in self.builder.secnumbers:
                     anchorname = ''  # try first heading which has no anchor
-            if self.builder.secnumbers.get(anchorname):
-                numbers = self.builder.secnumbers[anchorname]
-                self.body.append('.'.join(map(str, numbers)) +
-                                 self.secnumber_suffix)
 
-    def add_fignumber(self, node):
-        # type: (nodes.Element) -> None
-        def append_fignumber(figtype, figure_id):
-            # type: (str, str) -> None
+            if self.builder.secnumbers.get(anchorname):
+                return self.builder.secnumbers[anchorname]
+
+        return None
+
+    def add_secnumber(self, node: Element) -> None:
+        secnumber = self.get_secnumber(node)
+        if secnumber:
+            self.body.append('<span class="section-number">%s</span>' %
+                             ('.'.join(map(str, secnumber)) + self.secnumber_suffix))
+
+    def add_fignumber(self, node: Element) -> None:
+        def append_fignumber(figtype: str, figure_id: str) -> None:
             if self.builder.name == 'singlehtml':
                 key = "%s/%s" % (self.docnames[-1], figtype)
             else:
@@ -326,55 +304,47 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
             else:
                 append_fignumber(figtype, node['ids'][0])
 
-    def add_permalink_ref(self, node, title):
-        # type: (nodes.Element, str) -> None
+    def add_permalink_ref(self, node: Element, title: str) -> None:
         if node['ids'] and self.permalink_text and self.builder.add_permalinks:
             format = '<a class="headerlink" href="#%s" title="%s">%s</a>'
             self.body.append(format % (node['ids'][0], title, self.permalink_text))
 
     # overwritten
-    def visit_bullet_list(self, node):
-        # type: (nodes.Element) -> None
+    def visit_bullet_list(self, node: Element) -> None:
         if len(node) == 1 and isinstance(node[0], addnodes.toctree):
             # avoid emitting empty <ul></ul>
             raise nodes.SkipNode
         super().visit_bullet_list(node)
 
     # overwritten
-    def visit_definition(self, node):
-        # type: (nodes.Element) -> None
+    def visit_definition(self, node: Element) -> None:
         # don't insert </dt> here.
         self.body.append(self.starttag(node, 'dd', ''))
 
     # overwritten
-    def depart_definition(self, node):
-        # type: (nodes.Element) -> None
+    def depart_definition(self, node: Element) -> None:
         self.body.append('</dd>\n')
 
     # overwritten
-    def visit_classifier(self, node):
-        # type: (nodes.Element) -> None
+    def visit_classifier(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'span', '', CLASS='classifier'))
 
     # overwritten
-    def depart_classifier(self, node):
-        # type: (nodes.Element) -> None
+    def depart_classifier(self, node: Element) -> None:
         self.body.append('</span>')
 
-        next_node = node.next_node(descend=False, siblings=True)  # type: nodes.Node
+        next_node = node.next_node(descend=False, siblings=True)  # type: Node
         if not isinstance(next_node, nodes.classifier):
             # close `<dt>` tag at the tail of classifiers
             self.body.append('</dt>')
 
     # overwritten
-    def visit_term(self, node):
-        # type: (nodes.Element) -> None
+    def visit_term(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'dt', ''))
 
     # overwritten
-    def depart_term(self, node):
-        # type: (nodes.Element) -> None
-        next_node = node.next_node(descend=False, siblings=True)  # type: nodes.Node
+    def depart_term(self, node: Element) -> None:
+        next_node = node.next_node(descend=False, siblings=True)  # type: Node
         if isinstance(next_node, nodes.classifier):
             # Leave the end tag to `self.depart_classifier()`, in case
             # there's a classifier.
@@ -383,16 +353,14 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
             self.body.append('</dt>')
 
     # overwritten
-    def visit_title(self, node):
-        # type: (nodes.Element) -> None
+    def visit_title(self, node: Element) -> None:
         super().visit_title(node)
         self.add_secnumber(node)
         self.add_fignumber(node.parent)
         if isinstance(node.parent, nodes.table):
             self.body.append('<span class="caption-text">')
 
-    def depart_title(self, node):
-        # type: (nodes.Element) -> None
+    def depart_title(self, node: Element) -> None:
         close_tag = self.context[-1]
         if (self.permalink_text and self.builder.add_permalinks and
            node.parent.hasattr('ids') and node.parent['ids']):
@@ -414,8 +382,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         super().depart_title(node)
 
     # overwritten
-    def visit_literal_block(self, node):
-        # type: (nodes.Element) -> None
+    def visit_literal_block(self, node: Element) -> None:
         if node.rawsource != node.astext():
             # most probably a parsed-literal block -- don't highlight
             return super().visit_literal_block(node)
@@ -439,8 +406,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         self.body.append(starttag + highlighted + '</div>\n')
         raise nodes.SkipNode
 
-    def visit_caption(self, node):
-        # type: (nodes.Element) -> None
+    def visit_caption(self, node: Element) -> None:
         if isinstance(node.parent, nodes.container) and node.parent.get('literal_block'):
             self.body.append('<div class="code-block-caption">')
         else:
@@ -448,8 +414,7 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         self.add_fignumber(node.parent)
         self.body.append(self.starttag(node, 'span', '', CLASS='caption-text'))
 
-    def depart_caption(self, node):
-        # type: (nodes.Element) -> None
+    def depart_caption(self, node: Element) -> None:
         self.body.append('</span>')
 
         # append permalink if available
@@ -465,22 +430,18 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         else:
             super().depart_caption(node)
 
-    def visit_doctest_block(self, node):
-        # type: (nodes.Element) -> None
+    def visit_doctest_block(self, node: Element) -> None:
         self.visit_literal_block(node)
 
     # overwritten to add the <div> (for XHTML compliance)
-    def visit_block_quote(self, node):
-        # type: (nodes.Element) -> None
+    def visit_block_quote(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'blockquote') + '<div>')
 
-    def depart_block_quote(self, node):
-        # type: (nodes.Element) -> None
+    def depart_block_quote(self, node: Element) -> None:
         self.body.append('</div></blockquote>\n')
 
     # overwritten
-    def visit_literal(self, node):
-        # type: (nodes.Element) -> None
+    def visit_literal(self, node: Element) -> None:
         if 'kbd' in node['classes']:
             self.body.append(self.starttag(node, 'kbd', '',
                                            CLASS='docutils literal notranslate'))
@@ -489,16 +450,14 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
                                            CLASS='docutils literal notranslate'))
             self.protect_literal_text += 1
 
-    def depart_literal(self, node):
-        # type: (nodes.Element) -> None
+    def depart_literal(self, node: Element) -> None:
         if 'kbd' in node['classes']:
             self.body.append('</kbd>')
         else:
             self.protect_literal_text -= 1
             self.body.append('</code>')
 
-    def visit_productionlist(self, node):
-        # type: (nodes.Element) -> None
+    def visit_productionlist(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'pre'))
         names = []
         productionlist = cast(Iterable[addnodes.production], node)
@@ -518,37 +477,29 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         self.body.append('</pre>\n')
         raise nodes.SkipNode
 
-    def depart_productionlist(self, node):
-        # type: (nodes.Element) -> None
+    def depart_productionlist(self, node: Element) -> None:
         pass
 
-    def visit_production(self, node):
-        # type: (nodes.Element) -> None
+    def visit_production(self, node: Element) -> None:
         pass
 
-    def depart_production(self, node):
-        # type: (nodes.Element) -> None
+    def depart_production(self, node: Element) -> None:
         pass
 
-    def visit_centered(self, node):
-        # type: (nodes.Element) -> None
+    def visit_centered(self, node: Element) -> None:
         self.body.append(self.starttag(node, 'p', CLASS="centered") +
                          '<strong>')
 
-    def depart_centered(self, node):
-        # type: (nodes.Element) -> None
+    def depart_centered(self, node: Element) -> None:
         self.body.append('</strong></p>')
 
-    def visit_compact_paragraph(self, node):
-        # type: (nodes.Element) -> None
+    def visit_compact_paragraph(self, node: Element) -> None:
         pass
 
-    def depart_compact_paragraph(self, node):
-        # type: (nodes.Element) -> None
+    def depart_compact_paragraph(self, node: Element) -> None:
         pass
 
-    def visit_download_reference(self, node):
-        # type: (nodes.Element) -> None
+    def visit_download_reference(self, node: Element) -> None:
         atts = {'class': 'reference download',
                 'download': ''}
 
@@ -567,36 +518,17 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         else:
             self.context.append('')
 
-    def depart_download_reference(self, node):
-        # type: (nodes.Element) -> None
+    def depart_download_reference(self, node: Element) -> None:
         self.body.append(self.context.pop())
 
     # overwritten
-    def visit_image(self, node):
-        # type: (nodes.Element) -> None
+    def visit_image(self, node: Element) -> None:
         olduri = node['uri']
         # rewrite the URI if the environment knows about it
         if olduri in self.builder.images:
             node['uri'] = posixpath.join(self.builder.imgpath,
                                          self.builder.images[olduri])
 
-        uri = node['uri']
-        if uri.lower().endswith(('svg', 'svgz')):
-            atts = {'src': uri}
-            if 'width' in node:
-                atts['width'] = node['width']
-            if 'height' in node:
-                atts['height'] = node['height']
-            atts['alt'] = node.get('alt', uri)
-            if 'align' in node:
-                self.body.append('<div align="%s" class="align-%s">' %
-                                 (node['align'], node['align']))
-                self.context.append('</div>\n')
-            else:
-                self.context.append('')
-            self.body.append(self.emptytag(node, 'img', '', **atts))
-            return
-
         if 'scale' in node:
             # Try to figure out image height and width.  Docutils does that too,
             # but it tries the final file name, which does not necessarily exist
@@ -611,65 +543,71 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
                         node['width'] = str(size[0])
                     if 'height' not in node:
                         node['height'] = str(size[1])
+
+        uri = node['uri']
+        if uri.lower().endswith(('svg', 'svgz')):
+            atts = {'src': uri}
+            if 'width' in node:
+                atts['width'] = node['width']
+            if 'height' in node:
+                atts['height'] = node['height']
+            if 'scale' in node:
+                if 'width' in atts:
+                    atts['width'] = multiply_length(atts['width'], node['scale'])
+                if 'height' in atts:
+                    atts['height'] = multiply_length(atts['height'], node['scale'])
+            atts['alt'] = node.get('alt', uri)
+            if 'align' in node:
+                atts['class'] = 'align-%s' % node['align']
+            self.body.append(self.emptytag(node, 'img', '', **atts))
+            return
+
         super().visit_image(node)
 
     # overwritten
-    def depart_image(self, node):
-        # type: (nodes.Element) -> None
+    def depart_image(self, node: Element) -> None:
         if node['uri'].lower().endswith(('svg', 'svgz')):
-            self.body.append(self.context.pop())
+            pass
         else:
             super().depart_image(node)
 
-    def visit_toctree(self, node):
-        # type: (nodes.Element) -> None
+    def visit_toctree(self, node: Element) -> None:
         # this only happens when formatting a toc from env.tocs -- in this
         # case we don't want to include the subtree
         raise nodes.SkipNode
 
-    def visit_index(self, node):
-        # type: (nodes.Element) -> None
+    def visit_index(self, node: Element) -> None:
         raise nodes.SkipNode
 
-    def visit_tabular_col_spec(self, node):
-        # type: (nodes.Element) -> None
+    def visit_tabular_col_spec(self, node: Element) -> None:
         raise nodes.SkipNode
 
-    def visit_glossary(self, node):
-        # type: (nodes.Element) -> None
+    def visit_glossary(self, node: Element) -> None:
         pass
 
-    def depart_glossary(self, node):
-        # type: (nodes.Element) -> None
+    def depart_glossary(self, node: Element) -> None:
         pass
 
-    def visit_acks(self, node):
-        # type: (nodes.Element) -> None
+    def visit_acks(self, node: Element) -> None:
         pass
 
-    def depart_acks(self, node):
-        # type: (nodes.Element) -> None
+    def depart_acks(self, node: Element) -> None:
         pass
 
-    def visit_hlist(self, node):
-        # type: (nodes.Element) -> None
+    def visit_hlist(self, node: Element) -> None:
         self.body.append('<table class="hlist"><tr>')
 
-    def depart_hlist(self, node):
-        # type: (nodes.Element) -> None
+    def depart_hlist(self, node: Element) -> None:
         self.body.append('</tr></table>\n')
 
-    def visit_hlistcol(self, node):
-        # type: (nodes.Element) -> None
+    def visit_hlistcol(self, node: Element) -> None:
         self.body.append('<td>')
 
-    def depart_hlistcol(self, node):
-        # type: (nodes.Element) -> None
+    def depart_hlistcol(self, node: Element) -> None:
         self.body.append('</td>')
 
     # overwritten
-    def visit_Text(self, node):
-        # type: (nodes.Text) -> None
+    def visit_Text(self, node: Text) -> None:
         text = node.astext()
         encoded = self.encode(text)
         if self.protect_literal_text:
@@ -690,122 +628,95 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
                 encoded = self.cloak_email(encoded)
             self.body.append(encoded)
 
-    def visit_note(self, node):
-        # type: (nodes.Element) -> None
+    def visit_note(self, node: Element) -> None:
         self.visit_admonition(node, 'note')
 
-    def depart_note(self, node):
-        # type: (nodes.Element) -> None
+    def depart_note(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_warning(self, node):
-        # type: (nodes.Element) -> None
+    def visit_warning(self, node: Element) -> None:
         self.visit_admonition(node, 'warning')
 
-    def depart_warning(self, node):
-        # type: (nodes.Element) -> None
+    def depart_warning(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_attention(self, node):
-        # type: (nodes.Element) -> None
+    def visit_attention(self, node: Element) -> None:
         self.visit_admonition(node, 'attention')
 
-    def depart_attention(self, node):
-        # type: (nodes.Element) -> None
+    def depart_attention(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_caution(self, node):
-        # type: (nodes.Element) -> None
+    def visit_caution(self, node: Element) -> None:
         self.visit_admonition(node, 'caution')
 
-    def depart_caution(self, node):
-        # type: (nodes.Element) -> None
+    def depart_caution(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_danger(self, node):
-        # type: (nodes.Element) -> None
+    def visit_danger(self, node: Element) -> None:
         self.visit_admonition(node, 'danger')
 
-    def depart_danger(self, node):
-        # type: (nodes.Element) -> None
+    def depart_danger(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_error(self, node):
-        # type: (nodes.Element) -> None
+    def visit_error(self, node: Element) -> None:
         self.visit_admonition(node, 'error')
 
-    def depart_error(self, node):
-        # type: (nodes.Element) -> None
+    def depart_error(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_hint(self, node):
-        # type: (nodes.Element) -> None
+    def visit_hint(self, node: Element) -> None:
         self.visit_admonition(node, 'hint')
 
-    def depart_hint(self, node):
-        # type: (nodes.Element) -> None
+    def depart_hint(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_important(self, node):
-        # type: (nodes.Element) -> None
+    def visit_important(self, node: Element) -> None:
         self.visit_admonition(node, 'important')
 
-    def depart_important(self, node):
-        # type: (nodes.Element) -> None
+    def depart_important(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_tip(self, node):
-        # type: (nodes.Element) -> None
+    def visit_tip(self, node: Element) -> None:
         self.visit_admonition(node, 'tip')
 
-    def depart_tip(self, node):
-        # type: (nodes.Element) -> None
+    def depart_tip(self, node: Element) -> None:
         self.depart_admonition(node)
 
-    def visit_literal_emphasis(self, node):
-        # type: (nodes.Element) -> None
+    def visit_literal_emphasis(self, node: Element) -> None:
         return self.visit_emphasis(node)
 
-    def depart_literal_emphasis(self, node):
-        # type: (nodes.Element) -> None
+    def depart_literal_emphasis(self, node: Element) -> None:
         return self.depart_emphasis(node)
 
-    def visit_literal_strong(self, node):
-        # type: (nodes.Element) -> None
+    def visit_literal_strong(self, node: Element) -> None:
         return self.visit_strong(node)
 
-    def depart_literal_strong(self, node):
-        # type: (nodes.Element) -> None
+    def depart_literal_strong(self, node: Element) -> None:
         return self.depart_strong(node)
 
-    def visit_abbreviation(self, node):
-        # type: (nodes.Element) -> None
+    def visit_abbreviation(self, node: Element) -> None:
         attrs = {}
         if node.hasattr('explanation'):
             attrs['title'] = node['explanation']
         self.body.append(self.starttag(node, 'abbr', '', **attrs))
 
-    def depart_abbreviation(self, node):
-        # type: (nodes.Element) -> None
+    def depart_abbreviation(self, node: Element) -> None:
         self.body.append('</abbr>')
 
-    def visit_manpage(self, node):
-        # type: (nodes.Element) -> None
+    def visit_manpage(self, node: Element) -> None:
         self.visit_literal_emphasis(node)
         if self.manpages_url:
             node['refuri'] = self.manpages_url.format(**node.attributes)
             self.visit_reference(node)
 
-    def depart_manpage(self, node):
-        # type: (nodes.Element) -> None
+    def depart_manpage(self, node: Element) -> None:
         if self.manpages_url:
             self.depart_reference(node)
         self.depart_literal_emphasis(node)
 
     # overwritten to add even/odd classes
 
-    def generate_targets_for_table(self, node):
-        # type: (nodes.Element) -> None
+    def generate_targets_for_table(self, node: Element) -> None:
         """Generate hyperlink targets for tables.
 
         Original visit_table() generates hyperlink targets inside table tags
@@ -818,21 +729,22 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
             self.body.append('<span id="%s"></span>' % id)
             node['ids'].remove(id)
 
-    def visit_table(self, node):
-        # type: (nodes.Element) -> None
+    def visit_table(self, node: Element) -> None:
         self.generate_targets_for_table(node)
 
         self._table_row_index = 0
 
+        atts = {}
         classes = [cls.strip(' \t\n') for cls in self.settings.table_style.split(',')]
         classes.insert(0, "docutils")  # compat
         if 'align' in node:
             classes.append('align-%s' % node['align'])
-        tag = self.starttag(node, 'table', CLASS=' '.join(classes))
+        if 'width' in node:
+            atts['style'] = 'width: %s' % node['width']
+        tag = self.starttag(node, 'table', CLASS=' '.join(classes), **atts)
         self.body.append(tag)
 
-    def visit_row(self, node):
-        # type: (nodes.Element) -> None
+    def visit_row(self, node: Element) -> None:
         self._table_row_index += 1
         if self._table_row_index % 2 == 0:
             node['classes'].append('row-even')
@@ -841,75 +753,38 @@ class HTML5Translator(SphinxTranslator, BaseTranslator):
         self.body.append(self.starttag(node, 'tr', ''))
         node.column = 0  # type: ignore
 
-    def visit_field_list(self, node):
-        # type: (nodes.Element) -> None
+    def visit_field_list(self, node: Element) -> None:
         self._fieldlist_row_index = 0
         return super().visit_field_list(node)
 
-    def visit_field(self, node):
-        # type: (nodes.Element) -> None
+    def visit_field(self, node: Element) -> None:
         self._fieldlist_row_index += 1
         if self._fieldlist_row_index % 2 == 0:
             node['classes'].append('field-even')
         else:
             node['classes'].append('field-odd')
 
-    def visit_math(self, node, math_env=''):
-        # type: (nodes.Element, str) -> None
+    def visit_math(self, node: Element, math_env: str = '') -> None:
         name = self.builder.math_renderer_name
         visit, _ = self.builder.app.registry.html_inline_math_renderers[name]
         visit(self, node)
 
-    def depart_math(self, node, math_env=''):
-        # type: (nodes.Element, str) -> None
+    def depart_math(self, node: Element, math_env: str = '') -> None:
         name = self.builder.math_renderer_name
         _, depart = self.builder.app.registry.html_inline_math_renderers[name]
         if depart:
             depart(self, node)
 
-    def visit_math_block(self, node, math_env=''):
-        # type: (nodes.Element, str) -> None
+    def visit_math_block(self, node: Element, math_env: str = '') -> None:
         name = self.builder.math_renderer_name
         visit, _ = self.builder.app.registry.html_block_math_renderers[name]
         visit(self, node)
 
-    def depart_math_block(self, node, math_env=''):
-        # type: (nodes.Element, str) -> None
+    def depart_math_block(self, node: Element, math_env: str = '') -> None:
         name = self.builder.math_renderer_name
         _, depart = self.builder.app.registry.html_block_math_renderers[name]
         if depart:
             depart(self, node)
 
-    def unknown_visit(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_visit(self, node: Node) -> None:
         raise NotImplementedError('Unknown node: ' + node.__class__.__name__)
-
-    # --------- METHODS FOR COMPATIBILITY --------------------------------------
-
-    @property
-    def highlightlang(self):
-        # type: () -> str
-        warnings.warn('HTMLTranslator.highlightlang is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        return self.builder.config.highlight_language
-
-    @property
-    def highlightlang_base(self):
-        # type: () -> str
-        warnings.warn('HTMLTranslator.highlightlang_base is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        return self.builder.config.highlight_language
-
-    @property
-    def highlightopts(self):
-        # type: () -> str
-        warnings.warn('HTMLTranslator.highlightopts is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        return self.builder.config.highlight_options
-
-    @property
-    def highlightlinenothreshold(self):
-        # type: () -> int
-        warnings.warn('HTMLTranslator.highlightlinenothreshold is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        return sys.maxsize
